package eu.eudat.logic.services.forms;

import eu.eudat.models.data.user.components.commons.Rule;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * Created by ikalyvas on 3/5/2018.
 */
@Service("visibilityRuleService")
public class VisibilityRuleServiceImpl implements VisibilityRuleService {
    private Map<String, Boolean> elementVisibility = new HashMap<>();
    private VisibilityContext visibilityContext;
    private Map<String, Object> properties;

    public boolean isElementVisible(String id) {
        if (!this.elementVisibility.containsKey(id) || this.elementVisibility.get(id)) return true;
        return false;
    }

    public void setProperties(Map<String, Object> properties) {
        this.properties = properties;
        this.properties.entrySet().stream()
                .filter(stringObjectEntry -> stringObjectEntry.getValue() instanceof String && ((String) stringObjectEntry.getValue()).startsWith("[") && ((String) stringObjectEntry.getValue()).endsWith("]")).forEach(stringObjectEntry -> {
            stringObjectEntry.setValue(parseArray((String) stringObjectEntry.getValue()));
        });
    }

    private List<String> parseArray(String original) {
        String parsed = original.replace("[", "").replace("\"", "").replace("]", "");
        return Arrays.asList(parsed.split(","));
    }

    public void buildVisibilityContext(List<Rule> sources) {
        this.visibilityContext = new VisibilityContext();
        this.visibilityContext.buildVisibilityContext(sources);
        this.visibilityContext.getVisibilityRules().forEach(item-> this.evaluateVisibility(item));
    }

    private void evaluateVisibility(VisibilityRule rule) {
        List<VisibilityRuleSource> sources = rule.getVisibilityRuleSources();
        for(int i = 0; i < sources.size(); i++){
            if (properties.containsKey(sources.get(i).getVisibilityRuleSourceId()) && (isContained(properties.get(sources.get(i).getVisibilityRuleSourceId()), sources.get(i).getVisibilityRuleSourceValue()) || properties.get(sources.get(i).getVisibilityRuleSourceId()).equals(sources.get(i).getVisibilityRuleSourceValue()))) {
                this.elementVisibility.put(rule.getVisibilityRuleTargetId(), true);
            }else{
                this.elementVisibility.put(rule.getVisibilityRuleTargetId(), false);
                return;
            }
        }
    }

    private Boolean isContained(Object values, String source) {
        if (values instanceof List) {
            return ((Collection<?>) values).contains(source);
        }
        return false;
    }
}
